/******************************************************************************
 *
 * Copyright (c) 2003 Intel Corporation.  All rights reserved.
 *
 * Intel hereby grants you permission to copy, modify, and distribute this
 * software and its documentation.  Intel grants this permission provided
 * that the above copyright notice appears in all copies and that both the
 * copyright notice and this permission notice appear in supporting
 * documentation.  In addition, Intel grants this permission provided that
 * you prominently mark as not part of the original any modifications made
 * to this software or documentation, and that the name of Intel
 * Corporation not be used in advertising or publicity pertaining to the
 * software or the documentation without specific, written prior
 * permission.
 *
 * Intel provides this AS IS, WITHOUT ANY WARRANTY, INCLUDING THE WARRANTY
 * OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, and makes no
 * guarantee or representations regarding the use of, or the results of the
 * use of, the software and documentation in terms of correctness,
 * accuracy, reliability, currentness, or otherwise, and you rely on the
 * software, documentation, and results solely at your own risk.
 *
 **************************************************************************/
/*                                                                        */
/* History:                                                               */
/*  27Feb03  DDM  Initial Release                                         */
/*                                                                        */
/**************************************************************************/

#include "iop321.h"
#include <assert.h>

#define _Allocate_D_Cache_Line(MVA) \
asm("mcr\tp15, 0, %0, c7, c2, 5" : : "r" (MVA) : "memory")

#define _Invalidate_D_Cache() \
asm("mcr\tp15, 0, r0, c7, c6, 0" : : : "memory")

#define _Invalidate_D_TLB() \
asm("mcr\tp15, 0, r0, c8, c6, 0" : : : "memory")

#define _Drain_Write_Buffer() \
asm("mcr\tp15, 0, r0, c7, c10, 4")

#define _CPWAIT() \
asm("mrc\tp15, 0, r0, c2, c0, 0\n\t" \
	"mov\tr0, r0\n\t" \
	"sub\tpc, pc, #4" : : : "r0", "memory")

#define _Read_Translation_Table_Base() \
({ register unsigned _val_; \
   asm volatile("mrc\tp15, 0, %0, c2, c0, 0" : "=r" (_val_)); \
   _val_; \
})

#define _Write_PMNC(VAL) \
asm("mcr\tp14, 0, %0, c0, c0, 0" : : "r" (VAL))

#define _Read_CCNT() \
({ register unsigned _val_; \
   asm volatile("mrc\tp14, 0, %0, c1, c0, 0" : "=r" (_val_)); \
   _val_; \
})

#define _Read_AUX_Control() \
({ register unsigned _val_; \
   asm("mrc\tp15, 0, %0, c1, c0, 1" : "=r" (_val_)); \
   _val_ & 0x00000033; \
})

#define _Write_AUX_Control(VAL) \
asm("mcr\tp15, 0, %0, c1, c0, 1" : : "r" (VAL))

//
// These conversion macros assume RedBoot Standard Memory Map
//
#define PHYSICAL_TO_VIRTUAL(addr) ((addr) & ~0xE0000000)
#define VIRTUAL_TO_PHYSICAL(addr) ((addr) | 0xA0000000)

#define DCACHE_FLUSH_AREA 0xc0000000
static void clean_flush_whole_cache(void)
{
   int size = DCACHE_SIZE_TOTAL;
   unsigned long p = DCACHE_FLUSH_AREA;

   while (size)
   {
      _Allocate_D_Cache_Line(p);
      p += DCACHE_LINE_SIZE;
      size -= DCACHE_LINE_SIZE;
   }
   _Invalidate_D_Cache();
   _CPWAIT();
}

#define MMU_XBIT (1 << 12)
#define MMU_CBIT (1 << 3)
#define MMU_BBIT (1 << 2)
#define MMU_XCB_MASK (MMU_XBIT | MMU_CBIT | MMU_BBIT)
void cache_control(enum xscale_cache_policy policy, 
                   void *addr,
                   unsigned long size)
{
    unsigned long cur_addr = (unsigned long)addr;
    unsigned long *desc_section_entry;
    unsigned long ttb = _Read_Translation_Table_Base();
    unsigned mask;
    switch (policy) {
    case CACHE_POLICY_STALL_UNTIL_COMPLETE:
        mask = 0;
        break;
    case CACHE_POLICY_NO_CACHING_BUFFER_COALESCING:
        mask = MMU_BBIT;
        break;
    case CACHE_POLICY_WRITE_THROUGH_READ_ALLOCATE:
        mask = MMU_CBIT;
        break;
    case CACHE_POLICY_WRITE_BACK_READ_ALLOCATE:
        mask = MMU_CBIT | MMU_BBIT;
        break;
    case CACHE_POLICY_NO_CACHING_NO_BUFFER_COALESCING:
        mask = MMU_XBIT | MMU_BBIT;
        break;
    case CACHE_POLICY_MINI_DCACHE:
        mask = MMU_XBIT | MMU_CBIT;
        break;
    case CACHE_POLICY_WRITE_BACK_READ_WRITE_ALLOCATE:
        mask = MMU_XBIT | MMU_CBIT | MMU_BBIT;
        break;
    default:
        printf("Unknown cache policy %d\n", policy);
        return;
    }

    while (cur_addr < (unsigned long)addr + size)
    {
        desc_section_entry = (unsigned long *) 
          PHYSICAL_TO_VIRTUAL((ttb &~ 0x00003FFF) |
                              (cur_addr &~ 0x000FFFFF) >> 18);
        assert((*desc_section_entry & 0x3) == 2);
        *desc_section_entry &=~ MMU_XCB_MASK;
        *desc_section_entry |= mask;
        cur_addr+=SECTION_SIZE;
    }
    clean_flush_whole_cache();
    _Invalidate_D_TLB();
}

inline void timer_start()
{
    _Write_PMNC(0x00000707);
}

inline unsigned timer_read() 
{
    _Drain_Write_Buffer();
    _CPWAIT();
    return _Read_CCNT();
}

inline double ticks_to_mb_per_s(unsigned long ticks, unsigned long size)
{
    double d_size = ((double)size)/((double)(1024*1024));
    double d_sec = ((double)ticks)/((double)600000000);
    return d_size/d_sec;
}
