/******************************************************************************
 *
 * Copyright (c) 2003 Intel Corporation.  All rights reserved.
 *
 * Intel hereby grants you permission to copy, modify, and distribute this
 * software and its documentation.  Intel grants this permission provided
 * that the above copyright notice appears in all copies and that both the
 * copyright notice and this permission notice appear in supporting
 * documentation.  In addition, Intel grants this permission provided that
 * you prominently mark as not part of the original any modifications made
 * to this software or documentation, and that the name of Intel
 * Corporation not be used in advertising or publicity pertaining to the
 * software or the documentation without specific, written prior
 * permission.
 *
 * Intel provides this AS IS, WITHOUT ANY WARRANTY, INCLUDING THE WARRANTY
 * OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, and makes no
 * guarantee or representations regarding the use of, or the results of the
 * use of, the software and documentation in terms of correctness,
 * accuracy, reliability, currentness, or otherwise, and you rely on the
 * software, documentation, and results solely at your own risk.
 *
 **************************************************************************/
/*                                                                        */
/* History:                                                               */
/*  27Feb03  DDM  Initial Release                                         */
/*                                                                        */
/**************************************************************************/

#include <stdio.h>
#include <assert.h>
#include "iop321.h"

#define TEST_SIZE (8*1024*1024)

void memset_test(void *dest)
{
    timer_start();
    memset(dest, 0xff, TEST_SIZE);
    printf("memset(): Throughput = %3.2fMB/s\n", 
           ticks_to_mb_per_s(timer_read(), TEST_SIZE));
}

int main(void)
{
    unsigned long *dest = (unsigned long *)memalign(SECTION_SIZE, 
                                                    TEST_SIZE);

    assert (sizeof(unsigned char) == 1);
    assert (sizeof(unsigned short) == 2);
    assert (sizeof(unsigned long) == 4);

    cache_control(CACHE_POLICY_WRITE_BACK_READ_WRITE_ALLOCATE, 
                  dest, TEST_SIZE);
    printf("===============================\n");
    printf("Read/Write Line Allocate Policy\n");
    printf("===============================\n");
    memset_test(dest);

    cache_control(CACHE_POLICY_WRITE_BACK_READ_ALLOCATE,
                  dest, TEST_SIZE);
    printf("=========================\n");
    printf("Read Line Allocate Policy\n");
    printf("=========================\n");
    memset_test(dest);

    cache_control(CACHE_POLICY_WRITE_BACK_READ_WRITE_ALLOCATE,
                  dest, TEST_SIZE);

    free(dest);
}
