/******************************************************************************
 *
 * Copyright (c) 2003 Intel Corporation.  All rights reserved.
 *
 * Intel hereby grants you permission to copy, modify, and distribute this
 * software and its documentation.  Intel grants this permission provided
 * that the above copyright notice appears in all copies and that both the
 * copyright notice and this permission notice appear in supporting
 * documentation.  In addition, Intel grants this permission provided that
 * you prominently mark as not part of the original any modifications made
 * to this software or documentation, and that the name of Intel
 * Corporation not be used in advertising or publicity pertaining to the
 * software or the documentation without specific, written prior
 * permission.
 *
 * Intel provides this AS IS, WITHOUT ANY WARRANTY, INCLUDING THE WARRANTY
 * OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, and makes no
 * guarantee or representations regarding the use of, or the results of the
 * use of, the software and documentation in terms of correctness,
 * accuracy, reliability, currentness, or otherwise, and you rely on the
 * software, documentation, and results solely at your own risk.
 *
 **************************************************************************/
/*                                                                        */
/* History:                                                               */
/*  27Feb03  DDM  Initial Release                                         */
/*                                                                        */
/**************************************************************************/

#include <stdio.h>
#include <assert.h>
#include "iop321.h"

void my_memset_8bit(register unsigned char *dest,
                    register unsigned char c,
                    register unsigned long size)
{
    while (size)
    {
        *dest++ = c;
        size -= sizeof(c);
    }
}

void my_memset_16bit(register unsigned short *dest,
                     register unsigned short c,
                     register unsigned long size)
{
    while (size)
    {
        *dest++ = c;
        size -= sizeof(c);
    }
}

void my_memset_32bit(register unsigned long *dest,
                     register unsigned long c,
                     register unsigned long size)
{
    while (size)
    {
        *dest++ = c;
        size -= sizeof(c);
    }
}

#define TEST_SIZE (8*1024*1024)

void memset_test(void *dest)
{
    timer_start();
    my_memset_8bit((unsigned char *)dest, 
                   0xff,
                   TEST_SIZE);
    printf(" 8-bit memset(): Throughput = %3.2fMB/s\n", 
           ticks_to_mb_per_s(timer_read(), TEST_SIZE));

    timer_start();
    my_memset_16bit((unsigned short *)dest,
                    0xffff,
                    TEST_SIZE);
    printf("16-bit memset(): Throughput = %3.2fMB/s\n",
           ticks_to_mb_per_s(timer_read(), TEST_SIZE));


    timer_start();
    my_memset_32bit((unsigned long *)dest,
                    0xffffffff,
                    TEST_SIZE);
    printf("32-bit memset(): Throughput = %3.2fMB/s\n",
           ticks_to_mb_per_s(timer_read(), TEST_SIZE));
}

int main(void)
{
    unsigned long *dest = (unsigned long *)memalign(SECTION_SIZE,
                                                    TEST_SIZE);

    assert (sizeof(unsigned char) == 1);
    assert (sizeof(unsigned short) == 2);
    assert (sizeof(unsigned long) == 4);

    cache_control(CACHE_POLICY_NO_CACHING_NO_BUFFER_COALESCING,
                  dest, TEST_SIZE);
    printf("======================\n");
    printf("Coalescing is disabled\n");
    printf("======================\n");
    memset_test(dest);

    cache_control(CACHE_POLICY_NO_CACHING_BUFFER_COALESCING,
                  dest, TEST_SIZE);
    printf("=====================\n");
    printf("Coalescing is enabled\n");
    printf("=====================\n");
    memset_test(dest);

    cache_control(CACHE_POLICY_WRITE_BACK_READ_WRITE_ALLOCATE,
                  dest, TEST_SIZE);

    free(dest);
}
