/******************************************************************************
 *
 * Copyright (c) 2003 Intel Corporation.  All rights reserved.
 *
 * Intel hereby grants you permission to copy, modify, and distribute this
 * software and its documentation.  Intel grants this permission provided
 * that the above copyright notice appears in all copies and that both the
 * copyright notice and this permission notice appear in supporting
 * documentation.  In addition, Intel grants this permission provided that
 * you prominently mark as not part of the original any modifications made
 * to this software or documentation, and that the name of Intel
 * Corporation not be used in advertising or publicity pertaining to the
 * software or the documentation without specific, written prior
 * permission.
 *
 * Intel provides this AS IS, WITHOUT ANY WARRANTY, INCLUDING THE WARRANTY
 * OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, and makes no
 * guarantee or representations regarding the use of, or the results of the
 * use of, the software and documentation in terms of correctness,
 * accuracy, reliability, currentness, or otherwise, and you rely on the
 * software, documentation, and results solely at your own risk.
 *
 **************************************************************************/
/*                                                                        */
/* History:                                                               */
/*  27Feb03  DDM  Initial Release                                         */
/*                                                                        */
/**************************************************************************/

#include <stdio.h>
#include <assert.h>
#include "iop321.h"

#define TEST_SIZE (8*1024*1024)
#define MAX_WORK_LOAD 100
#define WORK_LOAD_INCR 1

inline void work(int workload, register unsigned long *src)
{
    volatile int i = 0;
    DELAY(workload);     // Simulate a variable workload
    i += src[0];
    i += src[2];
}

void test_with_pld(int workload, unsigned long *src)
{
    int num_iters = TEST_SIZE/DCACHE_LINE_SIZE - 1;
    while (num_iters--)
    {
        PRELOAD(NTH_CACHE_LINE(src,0));
        work(workload, src);
        src += DCACHE_LINE_SIZE;
    }
    work(workload, src); src += DCACHE_LINE_SIZE;
}

void test_without_pld(int workload, unsigned long *src)
{
    int num_iters = TEST_SIZE/DCACHE_LINE_SIZE - 0;
    while (num_iters--)
    {
        work(workload, src);
        src += DCACHE_LINE_SIZE;
    }
}

void pld_test_all(unsigned long *src)
{
    int work_load;
    int i, j;
    int no_pld_ticks, pld_ticks;
    FILE *fp = fopen("pld_test.csv", "w");
    if (fp == NULL)
        fprintf(stderr, "Error opening output file pld_test.csv");

    printf("+------+-------------+-------------+\n");
    printf("| Work | No Preloads |  Preloads   |\n");
    printf("+------+-------------+-------------+\n");
    for (work_load = 0; 
         work_load <= MAX_WORK_LOAD;
         work_load+=WORK_LOAD_INCR)
    {
        timer_start();
        test_without_pld(work_load, src);
        no_pld_ticks = timer_read();

        timer_start();
        test_with_pld(work_load, src);
        pld_ticks = timer_read();
        printf("| %4d | %11lu | %11lu |\n", 
               work_load, no_pld_ticks, pld_ticks);
        fprintf(fp, "%d,%lu,%lu\n", 
               work_load, no_pld_ticks, pld_ticks);
    }
    printf("+------+-------------+-------------+\n");
    fclose(fp);
}

int main(void)
{
    unsigned long *src = (unsigned long *)memalign(SECTION_SIZE,
                                                   TEST_SIZE);
    cache_control(CACHE_POLICY_WRITE_BACK_READ_WRITE_ALLOCATE,
                  src, TEST_SIZE);
    pld_test_all(src);
    free(src);
}
